@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2002 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: gerber.h,v 1.23 2008/01/08 00:01:13 thepurlieu Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
+/** \file gerber.h
+    \brief Header info for the RS274X parsing functions
+    \ingroup libgerbv
+*/
+
 #ifndef GERBER_H
 #define GERBER_H
 
@@ -30,16 +35,40 @@ extern "C" {
 
 #include <stdio.h>
 #include <glib.h>
-#include "gerb_image.h"
+
 #include "gerb_file.h"
 
+typedef struct gerb_state {
+    int curr_x;
+    int curr_y;
+    int prev_x;
+    int prev_y;
+    int delta_cp_x;
+    int delta_cp_y;
+    int curr_aperture;
+    int changed;
+    gerbv_aperture_state_t aperture_state;
+    gerbv_interpolation_t interpolation;
+    gerbv_interpolation_t prev_interpolation;
+    gerbv_net_t *parea_start_node;
+    gerbv_layer_t *layer;
+    gerbv_netstate_t *state;
+    int in_parea_fill;
+    int mq_on;
+} gerb_state_t;
+
 /*
  * parse gerber file pointed to by fd
  */
-gerb_image_t *parse_gerb(gerb_file_t *fd, gchar *directoryPath);
+gerbv_image_t *parse_gerb(gerb_file_t *fd, gchar *directoryPath);
 gboolean gerber_is_rs274x_p(gerb_file_t *fd, gboolean *returnFoundBinary);
 gboolean gerber_is_rs274d_p(gerb_file_t *fd);
+gerbv_net_t *
+gerber_create_new_net (gerbv_net_t *currentNet, gerbv_layer_t *layer, gerbv_netstate_t *state);
 
+gboolean
+gerber_create_new_aperture (gerbv_image_t *image, int *indexNumber,
+		gerbv_aperture_type_t apertureType, gdouble parameter1, gdouble parameter2);
 #ifdef __cplusplus
 }
 #endif
