@@ -4,7 +4,7 @@
  *
  *   Copyright (C) 2000-2003 Stefan Petersen (spe@stacken.kth.se)
  *
- * $Id: interface.h,v 1.3 2008/01/08 00:01:13 thepurlieu Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,34 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
+/** \file interface.h
+    \brief Header info for the GUI building functions for Gerber Viewer
+    \ingroup gerbv
+*/
+
+
 void
 interface_create_gui (int req_width, int req_height);
 
-void rename_main_window(char *filename, GtkWidget *main_win);
+void
+interface_set_render_type (int);
+
+void rename_main_window(char const* filename, GtkWidget* main_win);
 
 void
 set_window_icon (GtkWidget * this_window);
 
 gboolean
-interface_get_alert_dialog_response (gchar *primaryText, gchar *secondaryText);
+interface_get_alert_dialog_response (gchar *primaryText, 
+				     gchar *secondaryText,
+                                     gboolean show_checkbox, 
+				     gboolean *ask_to_show_again );
 
+void
+interface_show_alert_dialog (gchar *primaryText, 
+			     gchar *secondaryText, 
+			     gboolean show_checkbox, 
+			     gboolean *ask_to_show_again );
+
+void
+interface_show_modify_orientation_dialog (gerbv_user_transformation_t *transform, gerbv_unit_t screenUnit);
