@@ -19,10 +19,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
-
-/** @file pick-and-place.c
-    @contains the pick and place parser and renderer
- */ 
+/** \file pick-and-place.c
+    \brief PNP (pick-and-place) parsing functions
+    \ingroup libgerbv
+*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -54,19 +54,17 @@
 
 #include <assert.h>
 
+#include "gerbv.h"
 #include "gerber.h"
-#include "gerb_error.h"
-
 #ifdef RENDER_USING_GDK
   #include "draw-gdk.h"
 #else
   #include "draw.h"
 #endif
 
-#include "log.h"
-#include "setup.h"
 #include "csv.h"
 #include "pick-and-place.h"
+
 /* CHECKME - here gi18n is disabled */
 #define _(String) (String)
 
@@ -78,9 +76,92 @@
 /* DEBUG printing.  #define DEBUG 1 in config.h to use this fcn. */
 #define dprintf if(DEBUG) printf
 
+void gerb_transf_free(gerbv_transf_t *transf)
+{
+    g_free(transf);
+    
+}/*gerbv_transf_t*/
+
+
+void gerb_transf_reset(gerbv_transf_t* transf)
+{
+    memset(transf,0,sizeof(gerbv_transf_t));
+    
+    transf->r_mat[0][0] = transf->r_mat[1][1] = 1.0; /*off-diagonals 0 diagonals 1 */
+    //transf->r_mat[1][0] = transf->r_mat[0][1] = 0.0;
+    transf->scale = 1.0;
+    //transf->offset[0] = transf->offset[1] = 0.0;
+    
+} /*gerb_transf_reset*/
+
+
+gerbv_transf_t* gerb_transf_new(void)
+{
+    gerbv_transf_t *transf;
+    
+    transf = g_malloc(sizeof(gerbv_transf_t));
+    gerb_transf_reset(transf);
+    return transf;
+    
+
+} /*gerb_transf_new*/
+
+
+//!Rotation
+/*! append rotation to transformation.
+@param transf transformation to be modified
+@param angle in rad (counterclockwise rotation) */
+
+void gerb_transf_rotate(gerbv_transf_t* transf, double angle)
+{
+    double m[2][2]; 
+    double s = sin(angle), c = cos(angle);
+      
+    memcpy(m, transf->r_mat, sizeof(m));    
+    transf->r_mat[0][0] = c * m[0][0] - s * m[1][0];
+    transf->r_mat[0][1] = c * m[0][1] - s * m[1][1];
+    transf->r_mat[1][0] = s * m[0][0] + c * m[1][0];
+    transf->r_mat[1][1] = s * m[0][1] + c * m[1][1];
+//    transf->offset[0] = transf->offset[1] = 0.0; CHECK ME
+    
+} /*gerb_transf_rotate*/
+
+//!Translation
+/*! append translation to transformation.
+@param transf transformation to be modified
+@param shift_x translation in x direction
+@param shift_y translation in y direction */
+
+void gerb_transf_shift(gerbv_transf_t* transf, double shift_x, double shift_y)
+{
+      
+    transf->offset[0] += shift_x;
+    transf->offset[1] += shift_y;
+            
+} /*gerb_transf_shift*/
+
+void gerb_transf_apply(double x, double y, gerbv_transf_t* transf, double *out_x, double *out_y)
+{
+
+//    x += transf->offset[0];
+//    y += transf->offset[1];
+    *out_x = (x * transf->r_mat[0][0] + y * transf->r_mat[0][1]) * transf->scale;
+    *out_y = (x * transf->r_mat[1][0] + y * transf->r_mat[1][1]) * transf->scale;
+    *out_x += transf->offset[0];
+    *out_y += transf->offset[1];
+    
+    
+}/*gerb_transf_apply*/
+
+void
+pick_and_place_reset_bounding_box (gerbv_net_t *net) {
+	net->boundingBox.left = -HUGE_VAL;
+	net->boundingBox.right = HUGE_VAL;
+	net->boundingBox.bottom = -HUGE_VAL;
+	net->boundingBox.top = HUGE_VAL;
+}
+
 //! Parses a string representing float number with a unit, default is mil
-/** @param char a string to be screened for unit
-    @return a correctly converted double */
 static double 
 pick_and_place_get_float_unit(char *str)
 {
@@ -159,7 +240,7 @@ pick_and_place_parse_file(gerb_file_t *fd)
     char          *row[12];
     char          buf[MAXL+2], buf0[MAXL+2];
     double        tmp_x, tmp_y;
-    gerb_transf_t *tr_rot = gerb_transf_new();
+    gerbv_transf_t *tr_rot = gerb_transf_new();
     GArray 	*pnpParseDataArray = g_array_new (FALSE, FALSE, sizeof(PnpPartData));
     gboolean foundValidDataRow = FALSE;
     
@@ -293,8 +374,8 @@ pick_and_place_parse_file(gerb_file_t *fd)
 	    gerb_transf_apply( pnpPartData.pad_x -  pnpPartData.mid_x, 
 			       pnpPartData.pad_y - pnpPartData.mid_y, tr_rot, &tmp_x, &tmp_y);
 	    if ((fabs(tmp_y) > fabs(tmp_x/100)) && (fabs(tmp_x) > fabs(tmp_y/100))){
-		pnpPartData.length = 2 * tmp_x;/* get dimensions*/
-		pnpPartData.width = 2 * tmp_y;
+		pnpPartData.length = 2 * fabs(tmp_x);/* get dimensions*/
+		pnpPartData.width = 2 * fabs(tmp_y);
 		pnpPartData.shape = PART_SHAPE_STD;
 	    } else {
 		pnpPartData.length = 0.015;
@@ -456,14 +537,14 @@ pick_and_place_check_file_type(gerb_file_t *fd, gboolean *returnFoundBinary)
  *	Notes:
  *	------------------------------------------------------------------
  */
-gerb_image_t *
+gerbv_image_t *
 pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint boardSide) 
 {
-    gerb_image_t *image = NULL;
-    gerb_net_t *curr_net = NULL;
+    gerbv_image_t *image = NULL;
+    gerbv_net_t *curr_net = NULL;
     int i;
-    gerb_transf_t *tr_rot = gerb_transf_new();
-    drill_stats_t *stats;  /* Eventually replace with pick_place_stats */
+    gerbv_transf_t *tr_rot = gerb_transf_new();
+    gerbv_drill_stats_t *stats;  /* Eventually replace with pick_place_stats */
     gboolean foundElement = FALSE;
     
     /* step through and make sure we have an element on the layer before
@@ -481,19 +562,19 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
     if (!foundElement)
     	return NULL;
 
-    image = new_gerb_image(image);
+    image = gerbv_create_image(image, "Pick and Place (X-Y) File");
     if (image == NULL) {
 	GERB_FATAL_ERROR("malloc image failed\n");
     }
 	
-    image->format = (gerb_format_t *)g_malloc(sizeof(gerb_format_t));
+    image->format = (gerbv_format_t *)g_malloc(sizeof(gerbv_format_t));
     if (image->format == NULL) {
 	GERB_FATAL_ERROR("malloc format failed\n");
     }
-    memset((void *)image->format, 0, sizeof(gerb_format_t));
+    memset((void *)image->format, 0, sizeof(gerbv_format_t));
     
-    image->layertype = PICK_AND_PLACE;
-    stats = drill_stats_new();
+    image->layertype = GERBV_LAYERTYPE_PICKANDPLACE;
+    stats = gerbv_drill_stats_new();
     if (stats == NULL)
         GERB_FATAL_ERROR("malloc pick_place_stats failed\n");
     image->drill_stats = stats;
@@ -501,87 +582,101 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 
     curr_net = image->netlist;
     curr_net->layer = image->layers;
-    curr_net->state = image->states;	
+    curr_net->state = image->states;
+    pick_and_place_reset_bounding_box (curr_net);	
     image->info->min_x = HUGE_VAL;
     image->info->min_y = HUGE_VAL;
     image->info->max_x = -HUGE_VAL;
     image->info->max_y = -HUGE_VAL;
 
-    image->aperture[0] = (gerb_aperture_t *)g_malloc(sizeof(gerb_aperture_t));
-    memset((void *) image->aperture[0], 0, sizeof(gerb_aperture_t));
-    image->aperture[0]->type = CIRCLE;
+    image->aperture[0] = (gerbv_aperture_t *)g_malloc(sizeof(gerbv_aperture_t));
+    memset((void *) image->aperture[0], 0, sizeof(gerbv_aperture_t));
+    image->aperture[0]->type = GERBV_APTYPE_CIRCLE;
     image->aperture[0]->amacro = NULL;
-    image->aperture[0]->parameter[0] = 0.02;
+    image->aperture[0]->parameter[0] = 0.01;
     image->aperture[0]->nuf_parameters = 1;
 
     for (i = 0; i < parsedPickAndPlaceData->len; i++) {
 	PnpPartData partData = g_array_index(parsedPickAndPlaceData, PnpPartData, i);
-	float radius;                         
-	curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	float radius,labelOffset;  
+	                       
+	curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	curr_net = curr_net->next;
+	
 	assert(curr_net);
-	memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	curr_net->layer = image->layers;
-	curr_net->state = image->states;	
+	curr_net->state = image->states;
+	if ((partData.rotation > 89) && (partData.rotation < 91))
+		labelOffset = fabs(partData.length/2);
+	else if ((partData.rotation > 179) && (partData.rotation < 181))
+		labelOffset = fabs(partData.width/2);
+	else if ((partData.rotation > 269) && (partData.rotation < 271))
+		labelOffset = fabs(partData.length/2);
+	else if ((partData.rotation > -91) && (partData.rotation < -89))
+		labelOffset = fabs(partData.length/2);
+	else if ((partData.rotation > -181) && (partData.rotation < -179))
+		labelOffset = fabs(partData.width/2);
+	else if ((partData.rotation > -271) && (partData.rotation < -269))
+		labelOffset = fabs(partData.length/2);
+	else labelOffset = fabs(partData.width/2);
+	
 	partData.rotation *= M_PI/180; /* convert deg to rad */
 	/* check if the entry is on the specified layer */
 	if ((boardSide == 0) && !((partData.layer[0]=='b') || (partData.layer[0]=='B')))
 		continue;
 	if ((boardSide == 1) && !((partData.layer[0]=='t') || (partData.layer[0]=='T')))
-		continue;	
+		continue;
+
+	/* this first net is just a label holder, so calculate the lower
+	      left location to line up above the element */
+
+	curr_net->start_x = curr_net->stop_x = partData.mid_x;
+	curr_net->start_y = curr_net->stop_y = partData.mid_y + labelOffset + 0.01;   
+	curr_net->aperture = 0;
+	curr_net->aperture_state = GERBV_APERTURE_STATE_OFF;
+	curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
+	curr_net->layer = image->layers;
+	curr_net->state = image->states;
+	pick_and_place_reset_bounding_box (curr_net);
+
+	/* assign a label to this first draw primitive, in case we want
+	* to render some text next to the mark
+	*/
+	if (strlen (partData.designator) > 0) {
+	curr_net->label = g_string_new (partData.designator);
+	}
+
+	gerb_transf_reset(tr_rot);
+	gerb_transf_shift(tr_rot, partData.mid_x, partData.mid_y);
+	gerb_transf_rotate(tr_rot, -partData.rotation);
+	    
 	if ((partData.shape == PART_SHAPE_RECTANGLE) ||
 	    (partData.shape == PART_SHAPE_STD)) {
 	    // TODO: draw rectangle length x width taking into account rotation or pad x,y
-	    gerb_transf_reset(tr_rot);
+
+
 	    
-	    gerb_transf_shift(tr_rot, partData.mid_x, partData.mid_y);
-	    /* unrotate the part to make sure the label is in the same location */
-	    gerb_transf_rotate(tr_rot, -partData.rotation);
-	    /* this first net is just a label holder, so calculate the lower
-	       left location to line up above the element */
-	    gerb_transf_apply(-partData.length/2,partData.width/2, tr_rot, 
-			      &curr_net->start_x, &curr_net->start_y);
-	    gerb_transf_apply(-partData.length/2,partData.width/2, tr_rot, 
-			      &curr_net->stop_x, &curr_net->stop_y);
-	    /* re-rotate back to the correct orientation */
-	    gerb_transf_rotate(tr_rot, partData.rotation);
-	    
-	    curr_net->aperture = 0;
-	    curr_net->aperture_state = OFF;
-	    curr_net->interpolation = LINEARx1;
-	    curr_net->layer = image->layers;
-	    curr_net->state = image->states;
-	    
-	    /* assign a label to this first draw primitive, in case we want
-	     * to render some text next to the mark
-	     */
-	    if (strlen (partData.designator) > 0) {
-		curr_net->label = g_string_new (partData.designator);
-	    }
-	    
-	    /* rotate 180 to line up with PCB standard notation */
-	    gerb_transf_rotate(tr_rot, M_PI);
-	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    gerb_transf_apply(partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
 	    gerb_transf_apply(-partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->stop_x, &curr_net->stop_y);
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
 	    
-	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    
 	    gerb_transf_apply(-partData.length/2, partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
@@ -589,15 +684,16 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 			      &curr_net->stop_x, &curr_net->stop_y);
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
 	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    
 	    gerb_transf_apply(-partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
@@ -605,15 +701,16 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 			      &curr_net->stop_x, &curr_net->stop_y);
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
 	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    
 	    gerb_transf_apply(partData.length/2, -partData.width/2, tr_rot, 
 			      &curr_net->start_x, &curr_net->start_y);
@@ -621,15 +718,16 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 			      &curr_net->stop_x, &curr_net->stop_y);
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
 	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    
 	    if (partData.shape == PART_SHAPE_RECTANGLE) {
 		gerb_transf_apply(partData.length/4, -partData.width/2, tr_rot, 
@@ -643,43 +741,51 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 				  &curr_net->stop_x, &curr_net->stop_y);
 		
 		curr_net->aperture = 0;
-		curr_net->aperture_state = ON;
-		curr_net->interpolation = LINEARx1;
+		curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+		curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 		curr_net->layer = image->layers;
 		curr_net->state = image->states;
+		pick_and_place_reset_bounding_box (curr_net);
 		
-		curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+		curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 		curr_net = curr_net->next;
 		assert(curr_net);
-		memset((void *)curr_net, 0, sizeof(gerb_net_t));
+		memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 		gerb_transf_apply(partData.length/2, partData.width/4, tr_rot, 
 				  &curr_net->start_x, &curr_net->start_y);
 		gerb_transf_apply(partData.length/4, partData.width/4, tr_rot, 
 				  &curr_net->stop_x, &curr_net->stop_y);     
 	    }
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
+	    
 	    /* calculate a rough radius for the min/max screen calcs later */
 	    radius = max (partData.length/2, partData.width/2);
 	} else {
+	    gdouble tmp_x,tmp_y;
+	    
 	    curr_net->start_x = partData.mid_x;
 	    curr_net->start_y = partData.mid_y;
-	    curr_net->stop_x = partData.pad_x;
-	    curr_net->stop_y = partData.pad_y;
+	    gerb_transf_apply( partData.pad_x -  partData.mid_x, 
+			       partData.pad_y - partData.mid_y, tr_rot, &tmp_x, &tmp_y);
+			       
+	    curr_net->stop_x = tmp_x;
+	    curr_net->stop_y = tmp_y;
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = LINEARx1;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_LINEARx1;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
 	    
-	    curr_net->next = (gerb_net_t *)g_malloc(sizeof(gerb_net_t));
+	    curr_net->next = (gerbv_net_t *)g_malloc(sizeof(gerbv_net_t));
 	    curr_net = curr_net->next;
 	    assert(curr_net);
-	    memset((void *)curr_net, 0, sizeof(gerb_net_t));
+	    memset((void *)curr_net, 0, sizeof(gerbv_net_t));
 	    
 	    curr_net->start_x = partData.mid_x;
 	    curr_net->start_y = partData.mid_y;
@@ -687,13 +793,13 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
 	    curr_net->stop_y = partData.pad_y;
 	    
 	    curr_net->aperture = 0;
-	    curr_net->aperture_state = ON;
-	    curr_net->interpolation = CW_CIRCULAR;
+	    curr_net->aperture_state = GERBV_APERTURE_STATE_ON;
+	    curr_net->interpolation = GERBV_INTERPOLATION_CW_CIRCULAR;
 	    curr_net->layer = image->layers;
 	    curr_net->state = image->states;
+	    pick_and_place_reset_bounding_box (curr_net);
 	    
-	    curr_net->cirseg = (gerb_cirseg_t *)g_malloc(sizeof(gerb_cirseg_t));
-	    memset((void *)curr_net->cirseg, 0, sizeof(gerb_cirseg_t));
+	    curr_net->cirseg = g_new0 (gerbv_cirseg_t,1);
 	    curr_net->cirseg->angle1 = 0.0;
 	    curr_net->cirseg->angle2 = 360.0;
 	    curr_net->cirseg->cp_x = partData.mid_x;
@@ -731,8 +837,8 @@ pick_and_place_convert_pnp_data_to_image(GArray *parsedPickAndPlaceData, gint bo
  *	------------------------------------------------------------------
  */
 void
-pick_and_place_parse_file_to_images(gerb_file_t *fd, gerb_image_t **topImage,
-			gerb_image_t **bottomImage) 
+pick_and_place_parse_file_to_images(gerb_file_t *fd, gerbv_image_t **topImage,
+			gerbv_image_t **bottomImage) 
 { 
     GArray *parsedPickAndPlaceData = pick_and_place_parse_file (fd);
 
