@@ -2,7 +2,7 @@
  * tooltable.c
  * Copyright (C) 2004 dmitri (at) users.sourceforge.net
  *
- * $Id: tooltable.c,v 1.2 2008/01/15 17:38:19 danmc Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,13 +19,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
+/** \file tooltable.c
+    \brief Tool file parsing functions
+    \ingroup libgerbv
+*/
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "tooltable.h"
-
 #define MIN_TOOL_NUMBER         1       /* T01 */
 #define MAX_TOOL_NUMBER         99      /* T99 */
 
@@ -100,7 +103,7 @@ ProcessToolLine(const char *cp)
 
 
 int 
-ProcessToolsFile(const char *tf)
+gerbv_process_tools_file(const char *tf)
 {
     FILE *f;
     char buf[80];
@@ -125,15 +128,15 @@ ProcessToolsFile(const char *tf)
     fclose(f);
     have_tools_file = 1;
     return 1;
-} /* ProcessToolsFile */
+} /* gerbv_process_tools_file */
 
 
 double 
-GetToolDiameter_Inches(int toolNumber)
+gerbv_get_tool_diameter(int toolNumber)
 {
     if (!have_tools_file)
         return 0;
     if ((toolNumber < MIN_TOOL_NUMBER) || (toolNumber > MAX_TOOL_NUMBER))
         return 0;
     return tools[toolNumber];
-} /* GetToolDiameter_Inches */
+} /* gerbv_get_tool_diameter */
